const VERSION = 'app-v1'
const CACHES = [
  '/',
  '/index.html',
  '/icon.png',
  '/manifest.json',
  '/app.js',
  '/main.css',
]; // #1 这些默认缓存
self.addEventListener('install', function (event) {
  const filesUpdate = cache => cache.addAll(CACHES);
  event.waitUntil(caches.open(VERSION).then(filesUpdate))
})

self.addEventListener('activate', event => {
  // 执行激活过程，清除旧缓存（如果有）
  event.waitUntil(
    caches.keys()
      .then(function(cacheNames) {
        return Promise.all(
          cacheNames.filter(function(cacheName) {
            return cacheName !== VERSION;
          }).map(function(cacheName) {
            return caches.delete(cacheName);
          })
        );
      })
  );
});

self.addEventListener('fetch', event => {
  // #2 动态离线缓存可以在这里设置
  const url = new URL(event.request.url)
  let ignoreSearch = false
  if (url.pathname === '/') { // 解决 http://localhost:8080/?k=v 离线加载问题
    ignoreSearch = true
  }
  // 有网络请求网络，没网络从离线缓存取
  event.respondWith(
    caches.match(event.request, {ignoreSearch})
    .then(res => {
      if(res) {
        // 匹配 http://localhost:8080/    index.html不能缓存
        if(['/', '/service-worker.js'].includes(url.pathname)) {
          return fetch(event.request).then(res => {
            caches.open(VERSION).then(cache => cache.add(event.request))
            return res
          }).catch(e => {
            // 网络断开的情况从缓存取
            return res
          })
        }
        return res
      } else {
        return fetch(event.request).then(res => {
          // 匹配 静态资源 或  ajax get 请求
          const filename = url.pathname.split('/').pop()
          const ext = filename.split('.').pop()
          if ((event.request.method === 'GET') && filename.includes('.') && ['json', 'js', 'css'].includes(ext)) {
            caches.open(VERSION).then(cache => cache.add(event.request))
          }
          return res
        }).catch(e => {
          if (url.pathname === '/') {
            return
          }
          console.error('该资源service-worker未缓存且当前网络断开')
        })
      }
    })
  );
})
